package pipe.dataLayer;

import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Rectangle;

import javax.swing.JComponent;
import pipe.gui.*;

/**
 * <b>PetriNetObject</b> - Petri-Net Object Class<b> - <i>Abstract</i></b>
 * @see <p><a href="..\PNMLSchema\index.html">PNML  -  Petri-Net XMLSchema (stNet.xsd)</a>
 * @see </p><p><a href="..\..\..\UML\dataLayer.html">UML  -  PNML Package </a></p>
 * @version 1.0
 * @author James D Bloom
 */

public abstract class PetriNetObject extends JComponent implements Constants {

	protected final static int COMPONENT_DRAW_OFFSET = 5;
  /** Id */
  protected String id = null;
//  /** Color of PetriNetObject*/
//  protected Color color = null;
  /** Name Label for displaying name*/
  protected NameLabel pnName;
  protected Color objectColour = ELEMENT_LINE_COLOUR;
  protected Color selectionBorderColour = SELECTION_LINE_COLOUR;
  protected boolean selected = false;	// True if part of the current selection.
  protected boolean selectable = true;	// True if object can be selected.
  protected boolean draggable = true;	// True if object can be dragged.
  protected static boolean ignoreSelection = false;
  protected Rectangle bounds = new Rectangle();

  /**
	 * The ZoomController of the GuiView this component is part of.
	 */
	private ZoomController zoomControl;
	
	
	/**
   * Create PetriNetObject
   *
   * @param idInput Input Id
   * @param colorInput Input Color
   */
  public PetriNetObject(String idInput){
  	id = idInput;
  }

  /**
   * Create default PetriNetObject
   *
   */
  public PetriNetObject(){
  }
  
  /**
   * Set id
   *
   * @param idInput String value for id;
   */
  public void setId(String idInput) {
    id = idInput;
  }

  /**
   * Get id returns null if value not yet entered
   *
   * @return String value for id;
   */
  public String getId() {
    return id;
  }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);
  }

  /**
   * Returns Name Label - is used by GuiView
   * @return PetriNetObject's Name Label (Model View Controller Design Pattern)
   */
  public NameLabel getNameLabel(){
    return pnName;
  }

  public void addLabelToContainer() {
  	if (getParent() != null) {
  		getParent().add(pnName);
  	}
  }

  public boolean isSelected()
  {
  	return selected;
  }

  public void select()
  {
  	if (selectable && !selected)
  	{	
  		selected = true;
  		repaint();
  	}
  }

  public void deselect()
  {
  	if (selected)
  	{	
  		selected = false;
  		repaint();
  	}
  }

  public boolean isSelectable()
  {
  	return selectable;
  }

  public void setSelectable(boolean allow)
  {
  	selectable = allow;
  }

  public static void ignoreSelection(boolean ignore)
  {
  	ignoreSelection = ignore;
  }

  public boolean isDraggable()
  {
  	return draggable;
  }

  public void setDraggable(boolean allow)
  {
  	draggable = allow;
  }

  public void setObjectColour(Color c)
  {
  	objectColour = c;
  }

  public void setSelectionBorderColour(Color c)
  {
  	selectionBorderColour = c;
  }

  public void addedToGui() {
  }

  public void delete() {
	CreateGui.getModel().removePetriNetObject(this);
	removeFromContainer();
	removeAll();
  }

  public void removeFromContainer() {
  	Container c = getParent();
  	if (c!=null) c.remove(this);
  }
  
  public void addZoomController(ZoomController zoomControl2) {
	  zoomControl = zoomControl2;
  }

  protected ZoomController getZoomController(){
	  return zoomControl;
  }

}


