package pipe.dataLayer;

import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import pipe.gui.CreateGui;
import pipe.gui.Zoomable;

/**
 * <b>PlaceTransitionObject</b> - Petri-Net PLace or Transition SuperClass<b> - <i>Abstract</i></b>
 *
 * @see <p><a href="..\PNMLSchema\index.html">PNML  -  Petri-Net XMLSchema (stNet.xsd)</a>
 * @see </p><p><a href="..\..\..\UML\dataLayer.html">UML  -  PNML Package </a></p>
 * @version 1.0
 * @author James D Bloom
 * 
 * @author Edwin Chung 16 Mar 2007: modified the constructor and several
 * other functions so that DataLayer objects can be created outside the
 * GUI
 */

public abstract class PlaceTransitionObject extends PetriNetObject 
implements Zoomable{

	/** X-axis Position on screen*/
	protected double positionX;
	/** Y-axis Position on screen*/
	protected double positionY;

	protected double componentWidth;
	protected double componentHeight;

	/** Used in the mouse events to control dragging*/
	private boolean isDragging;

	private Collection connectTo = new LinkedList();
	private Collection connectFrom = new LinkedList();
	protected static Arc someArc;
		
	/**
	 * The "real" x coordinate of this place or transition in the net.
	 * i.e. the x position at 100% zoom.
	 */
	private double locationX;
	
	/**
	 * The "real" y coordinate of this place or transition in the net.
	 * i.e. the y position at 100% zoom.
	 */
	private double locationY;

	/**
	 * Create Petri-Net Object
	 *
	 * @param positionXInput X-axis Position
	 * @param positionYInput Y-axis Position
	 * @param idInput Place id
	 * @param nameInput Name
	 * @param nameOffsetXInput Name X-axis Position
	 * @param nameOffsetYInput Name Y-axis Position
	 * @param colorInput Color
	 */
	public PlaceTransitionObject(double positionXInput, double positionYInput, String idInput,
			String nameInput, double nameOffsetXInput, double nameOffsetYInput){

		this(positionXInput,positionYInput, idInput);
		setName(nameInput);
	}

	/**
	 * Create Petri-Net Object
	 *
	 * @param positionXInput X-axis Position
	 * @param positionYInput Y-axis Position
	 * @param idInput Place id
	 * @param colorInput Color
	 */
	public PlaceTransitionObject(double positionXInput, double positionYInput, String idInput){
		this(positionXInput,positionYInput);
		id = idInput;

	}

	/**
	 * Create Petri-Net Object
	 * This constructor does all the work, the others just call it.
	 *
	 * @param positionXInput X-axis Position
	 * @param positionYInput Y-axis Position
	 * @param colorInput Color
	 *
	 */
	public PlaceTransitionObject(double positionXInput, double positionYInput){
		if(CreateGui.getApp()!=null) addZoomController(CreateGui.getView().getZoomController());
		setPositionX(positionXInput);
		setPositionY(positionYInput);
		
//s		setBorder(BorderFactory.createLineBorder(Color.BLACK));

		/*sets up Namelabel for each PN object*/
		pnName = new NameLabel();
		pnName.setPosition((int)positionXInput,(int)(positionYInput+30));
	}

	/**
	 * Create empty Petri-Net Object
	 *
	 */
	public PlaceTransitionObject(){

	}

	/**
	 * Set X-axis position
	 *
	 * @param positionXInput Double value for X-axis position
	 */
	public void setPositionX(double positionXInput) {
		positionX = positionXInput;
		if (getZoomController() != null){
			locationX = getZoomController().getUnzoomedValue(positionX);
		} else {
			locationX = (int)positionX;
		}
	}

	/**
	 * Set Y-axis position
	 *
	 * @param positionYInput Double value for Y-axis position
	 */
	public void setPositionY(double positionYInput) {
		positionY = positionYInput;
		if (getZoomController() != null){
			locationY = getZoomController().getUnzoomedValue(positionY);
		} else {
			locationY = (int)positionY;
		}
	}

	/**
	 * Set id
	 *
	 * @param idInput String value for Place id;
	 */
	public void setId(String idInput) {
		id = idInput;
		setName(id);
//		System.out.println("setting id to: " + idInput);
	}

	/**
	 * Set name
	 *
	 * @param nameInput String value for Place name;
	 */
	public void setName(String nameInput) {
		//sets the text within the label
//		System.out.println("setting name to: " + nameInput);
		pnName.setText(nameInput);
		pnName.updateSize();

	}

	/**
	 * Get X-axis position
	 *
	 * @return Double value for X-axis position
	 */
	public double getPositionX() {
		return positionX;
	}

	/**
	 * Get Y-axis position
	 *
	 * @return Double value for Y-axis position
	 */
	public double getPositionY() {
		return positionY;
	}

	/**
	 * Get id
	 *
	 * @return String value for Place id;
	 */
	public String getId() {
		if(id != null)
			return id;
		else
			return pnName.getText();
	}

	/**
	 * Get name
	 *
	 * @return String value for Place name;
	 */
	public String getName() {
		if(pnName != null)
			return pnName.getText();
		else
			return id;
	}

	/**
	 * Get X-axis position
	 * returns null if value not yet entered
	 *
	 * @return Double value for X-axis position
	 */
	public Double getPositionXObject() {
		return new Double(locationX);
	}

	/**
	 * Get Y-axis position
	 * returns null if value not yet entered
	 *
	 * @return Double value for Y-axis position
	 *
	 */
	public Double getPositionYObject() {
		return new Double(locationY);
	}

	/** 
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
	}

	/** Returns the width bounds we want to use when initially creating the place on the gui
	 * @return Width bounds of Place
	 */
	public int boundsWidth() {
		return WIDTH+1;
	}

	/** Returns the height bounds we want to use when initially creating the place on the gui
	 * @return Height bounds of Place
	 */
	public int boundsHeight() {
		return HEIGHT+1;
	}

	/** Returns the distance between the outside of the component to the centre, in order
	 *  to position the centre of the place where the mouse clicks on the screen
	 * @return Top offset of Place
	 */
	public int centreOffsetTop() {
		double zoomBy = getZoomController().getPercent() * 0.01;
		return (int)(zoomBy * componentHeight/2.0);
	}

	/** Returns the distance between the outside of the component to the centre, in order
	 *  to position the centre of the place where the mouse clicks on the screen
	 * @return Left offset of Place
	 */
	public int centreOffsetLeft() {
		double zoomBy = getZoomController().getPercent() * 0.01;
		return (int)(zoomBy * componentWidth/2.0);
	}

	/** Calculates the BoundsOffsets used for setBounds() method */
	public void updateBounds(){
		int scaleFactor = 100;
		if (getZoomController() != null){
			scaleFactor = getZoomController().getPercent();
		}
		positionX = locationX * scaleFactor / 100.0;
		positionY = locationY * scaleFactor / 100.0;
		bounds.setBounds(
				(int)positionX,
				(int)positionY,
				(int)(componentWidth * scaleFactor / 100.0),
				(int)(componentHeight * scaleFactor / 100.0));
		bounds.grow(COMPONENT_DRAW_OFFSET, COMPONENT_DRAW_OFFSET);
		setBounds(bounds);
		
		/*updates Namelabel for zoomed object*/
		pnName.setPosition((int)positionX,(int)positionY+getHeight()+7);
	}

	/** Adds outwards arc to place/transition */
	public void addConnectTo(Arc newArc) {
		connectTo.add(newArc);
	}

	/** Adds inwards arc to place/transition */
	public void addConnectFrom(Arc newArc) {
		connectFrom.add(newArc);
	}

	public void removeFromArc(Arc oldArc) {
		connectFrom.remove(oldArc);
	}

	public void removeToArc(Arc oldArc) {
		connectTo.remove(oldArc);
	}

	/** Updates location of any attached arcs */
	public void updateConnected() {
		Iterator arcsFrom = connectFrom.iterator();
		while(arcsFrom.hasNext()) {
			someArc = ((Arc)arcsFrom.next());
			updateEndPoint(someArc);
			someArc.updateArcPosition();
		}

		Iterator arcsTo = connectTo.iterator();
		while(arcsTo.hasNext()) {
			someArc = ((Arc)arcsTo.next());
			updateEndPoint(someArc);
			someArc.updateArcPosition();
		}
	}

	/** Translates the component by x,y */
	public void translate(int x, int y) {
		setPositionX(positionX + x);
		setPositionY(positionY + y);
		updateBounds();
	}

	/** Sets the center of the component to position x, y */
	public void setCentre(double x,double y) {
		setPositionX(x - (getWidth()/2.0));
		setPositionY(y - (getHeight()/2.0));
		updateBounds();
		updateLabelLocation();
		updateConnected();
	}

	public Point2D.Double getCentre() {
		return new Point2D.Double(positionX + getWidth()/2.0,
				positionY + getHeight()/2.0);
	}

	private void updateLabelLocation() {
		pnName.setPosition((int)positionX,(int)(positionY+30));
	}

	public void delete() {
		getParent().remove(pnName);
		super.delete();
	}

	/** Handles selection for Place/Transitions */
	public void select() {
		if (selectable && !selected) {	
			super.select();
			Iterator arcsFrom = connectFrom.iterator();
			while(arcsFrom.hasNext()) {
				((Arc)arcsFrom.next()).select();
			}

			Iterator arcsTo = connectTo.iterator();
			while(arcsTo.hasNext()) {
				((Arc)arcsTo.next()).select();
			}
		} 	
	}

	public void addedToGui() {
		addLabelToContainer(); 	
	}

	public boolean areNotSameType(PlaceTransitionObject o)
	{
		return (((this instanceof Place) && (o instanceof Transition))
				|| ((this instanceof Transition ) && (o instanceof Place)));
	}

	public Iterator getConnectFromIterator() {
		return connectFrom.iterator();
	}

	public Iterator getConnectToIterator() {
		return connectTo.iterator();
	}

	abstract public void updateEndPoint(Arc arc);

	public void zoomUpdate() {
		updateBounds();
		updateConnected();
	}
}